@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import Callable, Dict, Generator, Iterator, List, Match, Optional, Set, Tuple
+from typing import Any, Callable, Dict, Generator, Iterator, List, Match, NamedTuple, Optional, Set, Tuple
 
 import os
 import re
@@ -9,8 +9,7 @@ from git_machete.exceptions import MacheteException
 from git_machete.utils import colored, debug, fmt
 from git_machete import utils
 from git_machete.constants import (
-    MAX_COUNT_FOR_INITIAL_LOG, EscapeCodes, SyncToRemoteStatuses,
-    GIT_FORMAT_PATTERNS)
+    GitFormatPatterns, MAX_COUNT_FOR_INITIAL_LOG, EscapeCodes, SyncToRemoteStatuses)
 
 
 class AnyRevision(str):
@@ -150,7 +149,20 @@ class ForkPointOverrideData:
         self.while_descendant_of_hash: FullCommitHash = while_descendant_of_hash
 
 
-ReflogEntry = Tuple[FullCommitHash, str]
+class GitLogEntry(NamedTuple):
+    hash: FullCommitHash
+    short_hash: ShortCommitHash
+    subject: str
+
+
+class GitReflogEntry(NamedTuple):
+    hash: FullCommitHash
+    reflog_subject: str
+
+
+class BranchPair(NamedTuple):
+    local_branch: LocalBranchShortName
+    local_or_remote_branch: AnyBranchName
 
 
 HEAD = AnyRevision.of("HEAD")
@@ -161,7 +173,8 @@ class GitContext:
     def __init__(self) -> None:
         self._git_version: Optional[Tuple[int, ...]] = None
         self._root_dir: Optional[str] = None
-        self._git_dir: Optional[str] = None
+        self._main_git_dir: Optional[str] = None
+        self._worktree_git_dir: Optional[str] = None
         self.__fetch_done_for: Set[str] = set()
         self.__config_cached: Optional[Dict[str, str]] = None
         self.__remotes_cached: Optional[List[str]] = None
@@ -174,22 +187,23 @@ class GitContext:
         self.__remote_branches_cached: Optional[List[RemoteBranchShortName]] = None
         self.__initial_log_shas_cached: Dict[FullCommitHash, List[FullCommitHash]] = {}
         self.__remaining_log_shas_cached: Dict[FullCommitHash, List[FullCommitHash]] = {}
-        self.__reflogs_cached: Optional[Dict[AnyBranchName, Optional[List[ReflogEntry]]]] = None
+        self.__reflogs_cached: Optional[Dict[AnyBranchName, Optional[List[GitReflogEntry]]]] = None
         self.__merge_base_cached: Dict[Tuple[FullCommitHash, FullCommitHash], FullCommitHash] = {}
         self.__contains_equivalent_tree_cached: Dict[Tuple[FullCommitHash, FullCommitHash], bool] = {}
 
     @staticmethod
-    def _run_git(git_cmd: str, *args: str, **kwargs: Dict[str, str]) -> int:
+    def _run_git(git_cmd: str, *args: str, **kwargs: Any) -> int:
         exit_code = utils.run_cmd("git", git_cmd, *args, **kwargs)
         if not kwargs.get("allow_non_zero") and exit_code != 0:
-            raise MacheteException(f"`{utils.get_cmd_shell_repr('git', git_cmd, *args, **kwargs)}` returned {exit_code}")
+            raise MacheteException(f"`{utils.get_cmd_shell_repr('git', git_cmd, *args, env=kwargs.get('env'))}` returned {exit_code}")
         return exit_code
 
     @staticmethod
-    def _popen_git(git_cmd: str, *args: str, **kwargs: Dict[str, str]) -> str:
+    def _popen_git(git_cmd: str, *args: str, **kwargs: Any) -> str:
+        allow_non_zero = kwargs.pop("allow_non_zero", False)
         exit_code, stdout, stderr = utils.popen_cmd("git", git_cmd, *args, **kwargs)
-        if not kwargs.get("allow_non_zero") and exit_code != 0:
-            exit_code_msg: str = fmt(f"`{utils.get_cmd_shell_repr('git', git_cmd, *args, **kwargs)}` returned {exit_code}\n")
+        if not allow_non_zero and exit_code != 0:
+            exit_code_msg: str = fmt(f"`{utils.get_cmd_shell_repr('git', git_cmd, *args, env=kwargs.get('env'))}` returned {exit_code}\n")
             stdout_msg: str = f"\n{utils.bold('stdout')}:\n{utils.dim(stdout)}" if stdout else ""
             stderr_msg: str = f"\n{utils.bold('stderr')}:\n{utils.dim(stderr)}" if stderr else ""
             # Not applying the formatter to avoid transforming whatever characters might be in the output of the command.
@@ -214,17 +228,17 @@ class GitContext:
         for name, fun in proposed_editor_funs:
             editor = fun()
             if not editor:
-                debug("get_default_editor()", f"'{name}' is undefined")
+                debug(f"'{name}' is undefined")
             else:
                 editor_repr = f"'{name}'{(' (' + editor + ')') if editor != name else ''}"
                 if not utils.find_executable(editor):
-                    debug("get_default_editor()", f"{editor_repr} is not available")
+                    debug(f"{editor_repr} is not available")
                     if name == "$" + git_machete_editor_var:
                         # In this specific case, when GIT_MACHETE_EDITOR is defined but doesn't point to a valid executable,
                         # it's more reasonable/less confusing to raise an error and exit without opening anything.
                         raise MacheteException(f"<b>{editor_repr}</b> is not available")
                 else:
-                    debug("get_default_editor()", f"{editor_repr} is available")
+                    debug(f"{editor_repr} is available")
                     if name != "$" + git_machete_editor_var and self.get_config_attr_or_none('advice.macheteEditorSelection') != 'false':
                         sample_alternative = 'nano' if editor.startswith('vi') else 'vi'
                         print(fmt(f"Opening <b>{editor_repr}</b>.\n",
@@ -252,23 +266,34 @@ class GitContext:
                 raise MacheteException("Not a git repository")
         return self._root_dir
 
-    def __get_git_dir(self) -> str:
-        if not self._git_dir:
+    def __get_worktree_git_dir(self) -> str:
+        if not self._worktree_git_dir:
+            try:
+                self._worktree_git_dir = self._popen_git("rev-parse", "--git-dir").strip()
+            except MacheteException:
+                raise MacheteException("Not a git repository")
+        return self._worktree_git_dir
+
+    def __get_main_git_dir(self) -> str:
+        if not self._main_git_dir:
             try:
                 git_dir: str = self._popen_git("rev-parse", "--git-dir").strip()
                 git_dir_parts = Path(git_dir).parts
                 if len(git_dir_parts) >= 3 and git_dir_parts[-3] == '.git' and git_dir_parts[-2] == 'worktrees':
-                    self._git_dir = os.path.join(*git_dir_parts[:-2])
-                    debug('__get_git_dir', f'git dir pointing to {git_dir} - we are in a worktree; '
-                                           f'using {self._git_dir} as the effective git dir instead')
+                    self._main_git_dir = os.path.join(*git_dir_parts[:-2])
+                    debug(f'git dir pointing to {git_dir} - we are in a worktree; '
+                          f'using {self._main_git_dir} as the effective git dir instead')
                 else:
-                    self._git_dir = git_dir
+                    self._main_git_dir = git_dir
             except MacheteException:
                 raise MacheteException("Not a git repository")
-        return self._git_dir
+        return self._main_git_dir
 
-    def get_git_subpath(self, *fragments: str) -> str:
-        return os.path.join(self.__get_git_dir(), *fragments)
+    def get_worktree_git_subpath(self, *fragments: str) -> str:
+        return os.path.join(self.__get_worktree_git_dir(), *fragments)
+
+    def get_main_git_subpath(self, *fragments: str) -> str:
+        return os.path.join(self.__get_main_git_dir(), *fragments)
 
     def get_git_timespec_parsed_to_unix_timestamp(self, date: str) -> int:
         try:
@@ -428,18 +453,21 @@ class GitContext:
         # Since many people don't use '--set-upstream' flag of 'push' or 'branch', we try to infer the remote if the tracking data is missing.
         return self.get_strict_counterpart_for_fetching_of_branch(branch) or self.__get_inferred_counterpart_for_fetching_of_branch(branch)
 
+    # Note that rebase/cherry-pick/merge/revert all happen on per-worktree basis,
+    # so we need to check .git/worktrees/<worktree>/<file> rather than .git/<file>
+
     def is_am_in_progress(self) -> bool:
         # As of git 2.24.1, this is how 'cmd_rebase()' in builtin/rebase.c checks whether am is in progress.
-        return os.path.isfile(self.get_git_subpath("rebase-apply", "applying"))
+        return os.path.isfile(self.get_worktree_git_subpath("rebase-apply", "applying"))
 
     def is_cherry_pick_in_progress(self) -> bool:
-        return os.path.isfile(self.get_git_subpath("CHERRY_PICK_HEAD"))
+        return os.path.isfile(self.get_worktree_git_subpath("CHERRY_PICK_HEAD"))
 
     def is_merge_in_progress(self) -> bool:
-        return os.path.isfile(self.get_git_subpath("MERGE_HEAD"))
+        return os.path.isfile(self.get_worktree_git_subpath("MERGE_HEAD"))
 
     def is_revert_in_progress(self) -> bool:
-        return os.path.isfile(self.get_git_subpath("REVERT_HEAD"))
+        return os.path.isfile(self.get_worktree_git_subpath("REVERT_HEAD"))
 
     def checkout(self, branch: LocalBranchShortName) -> None:
         self._run_git("checkout", "--quiet", branch, "--")
@@ -530,9 +558,9 @@ class GitContext:
             branch, pos = branch_and_pos
             if branch not in self.__reflogs_cached:
                 self.__reflogs_cached[AnyBranchName.of(branch)] = []
-            self.__reflogs_cached[AnyBranchName.of(branch)] += [(FullCommitHash.of(sha), subject)]
+            self.__reflogs_cached[AnyBranchName.of(branch)] += [GitReflogEntry(hash=FullCommitHash.of(sha), reflog_subject=subject)]
 
-    def get_reflog(self, branch: AnyBranchName) -> List[ReflogEntry]:
+    def get_reflog(self, branch: AnyBranchName) -> List[GitReflogEntry]:
         # git version 2.14.2 fixed a bug that caused fetching reflog of more than
         # one branch at the same time unreliable in certain cases
         if self.get_git_version() >= (2, 14, 2):
@@ -545,15 +573,17 @@ class GitContext:
             if branch not in self.__reflogs_cached:
                 # %H - full hash
                 # %gs - reflog subject
-                self.__reflogs_cached[branch] = [
-                    tuple(entry.split(":", 1)) for entry in utils.get_non_empty_lines(  # type: ignore
-                        # The trailing '--' is necessary to avoid ambiguity in case there is a file called just exactly like the branch 'branch'.
-                        self._popen_git("reflog", "show", "--format=%H:%gs", branch, "--"))
-                ]
+                self.__reflogs_cached[branch] = list(map(lambda x: GitReflogEntry(hash=FullCommitHash(x[0]), reflog_subject=x[1]),
+                                                         [entry.split(":", 1) for entry in utils.get_non_empty_lines(
+                                                             # The trailing '--' is necessary to avoid ambiguity in case there is a file called just exactly like the branch 'branch'.
+                                                             self._popen_git("reflog", "show", "--format=%H:%gs", branch, "--"))]
+                                                         ))
             return self.__reflogs_cached[branch]
 
-    def create_branch(self, branch: LocalBranchShortName, out_of_revision: AnyRevision) -> None:
-        self._run_git("checkout", "-b", branch, out_of_revision)
+    def create_branch(self, branch: LocalBranchShortName, out_of_revision: AnyRevision, switch_head: bool) -> None:
+        self._run_git("branch", branch, out_of_revision)
+        if switch_head:
+            self._run_git("checkout", branch)
         self.flush_caches()  # the repository state has changed because of a successful branch creation, let's defensively flush all the caches
 
     def flush_caches(self) -> None:
@@ -584,12 +614,12 @@ class GitContext:
 
         # .git/rebase-merge directory exists during cherry-pick-powered rebases,
         # e.g. all interactive ones and the ones where '--strategy=' or '--keep-empty' option has been passed
-        rebase_merge_head_name_file = self.get_git_subpath("rebase-merge", "head-name")
+        rebase_merge_head_name_file = self.get_worktree_git_subpath("rebase-merge", "head-name")
         if os.path.isfile(rebase_merge_head_name_file):
             head_name_file = rebase_merge_head_name_file
 
         # .git/rebase-apply directory exists during the remaining, i.e. am-powered rebases, but also during am sessions.
-        rebase_apply_head_name_file = self.get_git_subpath("rebase-apply", "head-name")
+        rebase_apply_head_name_file = self.get_worktree_git_subpath("rebase-apply", "head-name")
         # Most likely .git/rebase-apply/head-name can't exist during am sessions, but it's better to be safe.
         if not self.is_am_in_progress() and os.path.isfile(rebase_apply_head_name_file):
             head_name_file = rebase_apply_head_name_file
@@ -645,7 +675,10 @@ class GitContext:
             #   then there is exactly one merge-base - the ancestor,
             # * if neither of sha1, sha2 is an ancestor of another,
             #   then none of the (possibly more than one) merge-bases is equal to either of sha1/sha2 anyway.
-            self.__merge_base_cached[sha1, sha2] = FullCommitHash.of(self._popen_git("merge-base", sha1, sha2).rstrip())
+            # In the rare case when sha1, sha2 have no common commits, the flag: allow_non_zero=True
+            # (allows, non zero exit code to be returned by git merge-base command, without raising an exception)
+            # is used and the __get_merge_base function returns None.
+            self.__merge_base_cached[sha1, sha2] = FullCommitHash.of(self._popen_git("merge-base", sha1, sha2, allow_non_zero=True))
         return self.__merge_base_cached[sha1, sha2]
 
     # Note: the 'git rev-parse --verify' validation is not performed in case for either of earlier/later
@@ -682,10 +715,7 @@ class GitContext:
         if (earlier_commit_sha, later_commit_sha) in self.__contains_equivalent_tree_cached:
             return self.__contains_equivalent_tree_cached[earlier_commit_sha, later_commit_sha]
 
-        debug(
-            "contains_equivalent_tree",
-            f"earlier_revision={earlier_revision} later_revision={later_revision}",
-        )
+        debug()
 
         earlier_tree_sha = self.get_tree_sha_by_commit_sha(earlier_commit_sha)
 
@@ -717,14 +747,20 @@ class GitContext:
         return matching_remote_branches[0] if len(matching_remote_branches) == 1 else None
 
     def get_merged_local_branches(self) -> List[LocalBranchShortName]:
-        return list(map(
-            lambda branch: LocalBranchFullName.of(branch).to_short_name(),
-            utils.get_non_empty_lines(
-                self._popen_git("for-each-ref", "--format=%(refname)", "--merged", "HEAD", "refs/heads"))
-        ))
+        if self.get_git_version() >= (2, 7, 6):  # earliest version of git to support 'for-each-ref --merged'
+            return list(
+                map(lambda branch: LocalBranchFullName.of(branch).to_short_name(),
+                    utils.get_non_empty_lines(
+                        self._popen_git("for-each-ref", "--format=%(refname)", "--merged", "HEAD", "refs/heads"))))
+        else:
+            return list(
+                filter(lambda branch: self.is_ancestor_or_equal(branch, AnyRevision.of('HEAD')),
+                       map(lambda branch: LocalBranchFullName.of(branch).to_short_name(),
+                           utils.get_non_empty_lines(
+                               self._popen_git("for-each-ref", "--format=%(refname)", "refs/heads")))))
 
     def get_hook_path(self, hook_name: str) -> str:
-        hook_dir: str = self.get_config_attr_or_none("core.hooksPath") or self.get_git_subpath("hooks")
+        hook_dir: str = self.get_config_attr_or_none("core.hooksPath") or self.get_main_git_subpath("hooks")
         return os.path.join(hook_dir, hook_name)
 
     def check_hook_executable(self, hook_path: str) -> bool:
@@ -754,11 +790,13 @@ class GitContext:
 
     def rebase(self, onto: AnyRevision, fork_revision: AnyRevision, branch: LocalBranchShortName, opt_no_interactive_rebase: bool) -> None:
         def do_rebase() -> None:
+            # Let's use `OPTS` suffix for consistency with git's built-in env var `GIT_DIFF_OPTS`
+            git_machete_rebase_opts_var = 'GIT_MACHETE_REBASE_OPTS'
+            rebase_opts = os.environ.get(git_machete_rebase_opts_var, '').split()
             try:
-                if opt_no_interactive_rebase:
-                    self._run_git("rebase", "--onto", onto, fork_revision, branch)
-                else:
-                    self._run_git("rebase", "--interactive", "--onto", onto, fork_revision, branch)
+                if not opt_no_interactive_rebase:
+                    rebase_opts.append("--interactive")
+                self._run_git("rebase", *rebase_opts, "--onto", onto, fork_revision, branch)
             finally:
                 # https://public-inbox.org/git/317468c6-40cc-9f26-8ee3-3392c3908efb@talktalk.net/T
                 # In our case, this can happen when git version invoked by git-machete to start the rebase
@@ -771,7 +809,7 @@ class GitContext:
 
                 # No need to fix <git-dir>/rebase-apply/author-script,
                 # only <git-dir>/rebase-merge/author-script (i.e. interactive rebases, for the most part) is affected.
-                author_script = self.get_git_subpath("rebase-merge", "author-script")
+                author_script = self.get_worktree_git_subpath("rebase-merge", "author-script")
                 if os.path.isfile(author_script):
                     faulty_line_regex = re.compile("[A-Z0-9_]+='[^']*")
 
@@ -789,7 +827,7 @@ class GitContext:
 
         hook_path = self.get_hook_path("machete-pre-rebase")
         if self.check_hook_executable(hook_path):
-            debug(f"rebase({onto}, {fork_revision}, {branch})", f"running machete-pre-rebase hook ({hook_path})")
+            debug(f"running machete-pre-rebase hook ({hook_path})")
             exit_code = utils.run_cmd(hook_path, onto, fork_revision, branch, cwd=self.get_root_dir())
             if exit_code == 0:
                 do_rebase()
@@ -802,12 +840,13 @@ class GitContext:
     def rebase_onto_ancestor_commit(self, branch: LocalBranchShortName, ancestor_revision: AnyRevision, opt_no_interactive_rebase: bool) -> None:
         self.rebase(ancestor_revision, ancestor_revision, branch, opt_no_interactive_rebase)
 
-    def get_commits_between(self, earliest_exclusive: AnyRevision, latest_inclusive: AnyRevision) -> List[Tuple[str, str, str]]:
+    def get_commits_between(self, earliest_exclusive: AnyRevision, latest_inclusive: AnyRevision) -> List[GitLogEntry]:
         # Reverse the list, since `git log` by default returns the commits from the latest to earliest.
         return list(reversed(list(map(
-            lambda x: tuple(x.split(":", 2)),  # type: ignore
-            utils.get_non_empty_lines(
-                self._popen_git("log", "--format=%H:%h:%s", f"^{earliest_exclusive}", latest_inclusive, "--"))
+            lambda x: GitLogEntry(hash=FullCommitHash(x.split(":", 2)[0]),
+                                  short_hash=ShortCommitHash(x.split(":", 2)[1]),
+                                  subject=x.split(":", 2)[2]),
+            utils.get_non_empty_lines(self._popen_git("log", "--format=%H:%h:%s", f"^{earliest_exclusive}", latest_inclusive, "--"))
         ))))
 
     def get_relation_to_remote_counterpart(self, branch: LocalBranchShortName, remote_branch: RemoteBranchShortName) -> int:
@@ -859,19 +898,19 @@ class GitContext:
                     result[to_branch] = int(match.group(1))
         return result
 
-    def get_commit_information(self, information: str, commit: Optional[AnyRevision] = None) -> str:
-        if information not in GIT_FORMAT_PATTERNS:
+    def get_commit_information(self, commit: AnyRevision, information: GitFormatPatterns) -> str:
+        if information not in GitFormatPatterns:
             raise MacheteException(
                 f"Retrieving {information} from commit is not supported by project"
                 " git-machete. Currently supported information are: "
-                f"{', '.join(GIT_FORMAT_PATTERNS.keys())}")
+                f"{', '.join(GitFormatPatterns._member_names_)}")
 
-        params = ["log", "-1", f"--format={GIT_FORMAT_PATTERNS[information]}"]
+        params = ["log", "-1", f"--format={information.value}"]
         if commit:
             params.append(commit)
         return self._popen_git(*params)
 
-    def display_branch_history_from_forkpoint(self, branch: LocalBranchFullName, fork_point: FullCommitHash) -> int:
+    def display_branch_history_from_fork_point(self, branch: LocalBranchFullName, fork_point: FullCommitHash) -> int:
         return self._run_git("log", f"^{fork_point}", branch)
 
     def commit_tree_with_given_parent_and_message_and_env(
