@@ -1,4 +1,4 @@
-
+import inspect
 from typing import Any, Callable, Dict, Iterable, List, Optional, Set, Tuple, TypeVar
 
 import os
@@ -87,7 +87,7 @@ def find_executable(executable: str) -> Optional[str]:
     for p in paths:
         f = os.path.join(p, executable)
         if os.path.isfile(f) and is_executable(f):
-            debug(f"find_executable({executable})", f"found {executable} at {f}")
+            debug(f"found {executable} at {f}")
             return f
     return None
 
@@ -154,15 +154,21 @@ def get_pretty_choices(*choices: str) -> str:
     return f" ({', '.join(map_truthy_only(format_choice, choices))}) "
 
 
-def debug(hdr: str, msg: str) -> None:
+def debug(msg: Optional[str] = None) -> None:
     if debug_mode:
-        print(f"{bold(hdr)}: {dim(msg)}", file=sys.stderr)
+        function_name = bold(inspect.stack()[1].function)
+        args, _, _, values = inspect.getargvalues(inspect.stack()[1].frame)
+        function_args = bold(f"({', '.join([arg+'='+str(values[arg]) if arg != 'self' else 'self' for arg in args])})")
+        if msg is None:
+            print(f"{function_name}{function_args}", file=sys.stderr)
+        else:
+            print(f"{function_name}{function_args}: {dim(msg)}", file=sys.stderr)
 
 
 def run_cmd(cmd: str, *args: str, **kwargs: Any) -> int:
     chdir_upwards_until_current_directory_exists()
 
-    flat_cmd: str = get_cmd_shell_repr(cmd, *args, **kwargs)
+    flat_cmd: str = get_cmd_shell_repr(cmd, *args, env=kwargs.get('env'))
     if debug_mode:
         print(bold(f">>> {flat_cmd}"), file=sys.stderr)
     elif verbose_mode:
@@ -195,15 +201,14 @@ def chdir_upwards_until_current_directory_exists() -> None:
                 # it doesn't propagate to the parent process (which is typically a shell).
                 os.chdir(os.path.pardir)
                 current_directory = get_current_directory_or_none()
-            debug("chdir_upwards_until_current_directory_exists()",
-                  f"current directory did not exist, chdired up into {current_directory}")
+            debug(f"current directory did not exist, chdired up into {current_directory}")
         current_directory_confirmed_to_exist = True
 
 
 def popen_cmd(cmd: str, *args: str, **kwargs: Any) -> Tuple[int, str, str]:
     chdir_upwards_until_current_directory_exists()
 
-    flat_cmd = get_cmd_shell_repr(cmd, *args, **kwargs)
+    flat_cmd = get_cmd_shell_repr(cmd, *args, env=kwargs.get('env'))
     if debug_mode:
         print(bold(f">>> {flat_cmd}"), file=sys.stderr)
     elif verbose_mode:
@@ -226,7 +231,7 @@ def popen_cmd(cmd: str, *args: str, **kwargs: Any) -> Tuple[int, str, str]:
     return exit_code, stdout, stderr
 
 
-def get_cmd_shell_repr(cmd: str, *args: str, **kwargs: Dict[str, str]) -> str:
+def get_cmd_shell_repr(cmd: str, *args: str, env: Optional[Dict[str, str]]) -> str:
     def shell_escape(arg: str) -> str:
         return arg.replace("(", "\\(") \
             .replace(")", "\\)") \
@@ -234,7 +239,7 @@ def get_cmd_shell_repr(cmd: str, *args: str, **kwargs: Dict[str, str]) -> str:
             .replace("\t", "$'\\t'") \
             .replace("\n", "$'\\n'")
 
-    env: Dict[str, str] = kwargs.get("env", {})
+    env = env if env is not None else {}
     # We don't want to include the env vars that are inherited from the environment of git-machete process
     env_repr = [k + "=" + shell_escape(v) for k, v in env.items() if k not in os.environ]
     return " ".join(env_repr + [cmd] + list(map(shell_escape, args)))
